/* DSP LABORATORY 
 * University of Technology Graz, 2018
 * Josef Kulmer
 * 
 * A FIRfilter objects performs the convolution of the provided input signal
 * using internal coefficients, based on block processing.
 */

#include <vector>
#include <math.h>
#include "blockprocessing.h"
#include "iir2.h"

using std::vector;

iir2::iir2() {
    // constructor
    x.resize(FRAMELENGTH);
    y.resize(FRAMELENGTH);
    bufferin.resize(2);
    bufferout.resize(2);
};

iir2::~iir2() { //deconstructor
};

// set initial coefficients 
void iir2::setCoefficients() {
    a0 = Q15;
    a1 = 0; 
    a2 = 0;
    b0 = Q15;  
    b1 = 0;
    b2 = 0;
    
    bufferout[0] = 0;    // = y(-1)
    bufferout[1] = 0;    // = y(-2)
};

// Biquad filter implementation using Direct form 1
void iir2::doFiltering(const void *in, void *out) {
    float *input = (float*) in;
    float *output = (float*) out;

    for (int i = 0; i < FRAMELENGTH; i++) {
        x[i] = static_cast<short>(Q15*input[i]); 
        y[i] = 0;
    }
    
    // first element
    y[0] = (b0 * (x[0]) + b1 * bufferin[0] + b2 * bufferin[1] - a1 * bufferout[0] - a2 * bufferout[1])/a0;
    // second element
    y[1] = (b0 * (x[1]) + b1 * (x[0]) + b2 * bufferin[0] - a1 * (y[0]) - a2 * bufferout[0])/a0;
    
    // for all other elements
    for (int i = 2; i < FRAMELENGTH; i++) {
        y[i] = (b0 * (x[i]) + b1 * (x[i-1]) + b2 * (x[i-2]) - a1 * (y[i-1]) - a2 * (y[i-2]))/a0;
    }

    // finally save last samples
    bufferin[0]=x[FRAMELENGTH-1]; 
    bufferin[1]=x[FRAMELENGTH-2];
    bufferout[0]=y[FRAMELENGTH-1];
    bufferout[1]=y[FRAMELENGTH-2];
    
    for (int i = 0; i<FRAMELENGTH; i++) {
        output[i] = static_cast<float>(y[i])/Q15;
    }
}
